from django.db import models
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
class Hit(models.Model):
    path = models.CharField(max_length=500, db_index=True)
    content_type = models.ForeignKey(ContentType, null=True, blank=True, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField(null=True, blank=True)
    content_object = GenericForeignKey("content_type", "object_id")
    hits = models.PositiveBigIntegerField(default=0)
    last_hit = models.DateTimeField(auto_now=True)
    class Meta:
        unique_together = (("path", "content_type", "object_id"),)
        indexes = [models.Index(fields=["path"])]
    def __str__(self):
        if self.content_object:
            return f"Hits for {self.content_object} ({self.hits})"
        return f"Hits for {self.path} ({self.hits})"



# Generated by Django 5.2.7 on 2025-10-25 15:05

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Hit',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.CharField(db_index=True, max_length=500)),
                ('object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('hits', models.PositiveBigIntegerField(default=0)),
                ('last_hit', models.DateTimeField(auto_now=True)),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'indexes': [models.Index(fields=['path'], name='django_page_path_c76430_idx')],
                'unique_together': {('path', 'content_type', 'object_id')},
            },
        ),
    ]



from django.utils.deprecation import MiddlewareMixin
from django.conf import settings
from django.db import models as dj_models
from .models import Hit
from django.contrib.contenttypes.models import ContentType
import re
EXCLUDE_PATHS = getattr(settings, "PAGE_HIT_COUNTER_EXCLUDE_PATHS", [r"^/admin/", r"^/static/", r"^/media/", r"^/__debug__/"])
class PageHitCounterMiddleware(MiddlewareMixin):
    def process_request(self, request):
        if request.method != "GET":
            return None
        path = request.path
        for patt in EXCLUDE_PATHS:
            if re.match(patt, path):
                return None
        if getattr(settings, "PAGE_HIT_COUNTER_IGNORE_AJAX", True):
            if request.headers.get("x-requested-with") == "XMLHttpRequest":
                return None
        hit_obj = getattr(request, "hit_object", None)
        if hit_obj is not None:
            ct = ContentType.objects.get_for_model(type(hit_obj))
            obj_id = getattr(hit_obj, "pk", None)
            hit, _ = Hit.objects.get_or_create(path=path, content_type=ct, object_id=obj_id)
        else:
            hit, _ = Hit.objects.get_or_create(path=path, content_type=None, object_id=None)
        Hit.objects.filter(pk=hit.pk).update(hits=dj_models.F("hits") + 1)
        return None



from django import template
from ..models import Hit
from django.contrib.contenttypes.models import ContentType
register = template.Library()
@register.simple_tag(takes_context=True)
def get_hits_for_path(context, path=None):
    if path is None:
        path = context.get("request").path
    try:
        hit = Hit.objects.get(path=path, content_type=None, object_id=None)
        return hit.hits
    except Hit.DoesNotExist:
        return 0
@register.simple_tag
def get_hits_for_object(obj):
    ct = ContentType.objects.get_for_model(type(obj))
    try:
        hit = Hit.objects.get(path=obj.get_absolute_url() if hasattr(obj, "get_absolute_url") else "", content_type=ct, object_id=obj.pk)
        return hit.hits
    except Hit.DoesNotExist:
        return 0
@register.inclusion_tag("hit_counter/hits.html", takes_context=True)
def render_hits(context, path=None, obj=None, label="Views:"):
    if obj is not None:
        hits = get_hits_for_object(obj)
    else:
        hits = get_hits_for_path(context, path)
    return {"hits": hits, "label": label}



from django.contrib import admin
from .models import Hit
@admin.register(Hit)
class HitAdmin(admin.ModelAdmin):
    list_display = ("path", "content_type", "object_id", "hits", "last_hit")
    search_fields = ("path",)
    readonly_fields = ("last_hit",)



from setuptools import setup, find_packages
import pathlib
HERE = pathlib.Path(__file__).parent
README = (HERE / 'README.md').read_text()
setup(
    name='django_page_hit_counter',
    version='0.1',
    description='Simple Django page hit counter',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/drnimishadavis/djangopagehitcounter',
    author='Nimisha Davis',
    license='MIT',
    packages=find_packages(exclude=('tests',)),
    include_package_data=True,
    install_requires=['Django>=3.2'],
    classifiers=['Framework :: Django','Programming Language :: Python :: 3'],
)



from pathlib import Path
BASE_DIR = Path(__file__).resolve().parent.parent
SECRET_KEY = "replace-this-with-a-secure-key"
DEBUG = True
ALLOWED_HOSTS = []
INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "demo_pages",
    "django_page_hit_counter",
]
MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django_page_hit_counter.middleware.PageHitCounterMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]
ROOT_URLCONF = "demo_project.urls"
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [ BASE_DIR / "templates" ],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]
WSGI_APPLICATION = "demo_project.wsgi.application"
DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": BASE_DIR / "db.sqlite3"}}
AUTH_PASSWORD_VALIDATORS = []
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_TZ = True
STATIC_URL = "/static/"
